/*
	Code generation for the following grammar segment:

	CompilationUnit :=
		[ModuleDeclaration()]
		(ImportDeclaration())*
		[BehaviorDeclarationAttributes()]
		( BehaviorDeclaration() | InterfaceDeclaration() )
 */

package salsac.definitions;

import salsac.*;

public class CompilationUnit extends SimpleNode {
	public CompilationUnit(int id) 			{ super(id); }
	public CompilationUnit(SalsaParser p, int id)	{ super(p, id); }

	public String getModule() {
		if (getChild(0) instanceof ModuleDeclaration) return ((ModuleDeclaration)getChild(0)).getModule();
		else return "";
	}

	public String getChildCode() {
		int place = 0;

		String code = "";
		if (getChild(0) instanceof ModuleDeclaration) {
			code += getChild(0).getJavaCode();
			place++;
		}

		code += "// Import declarations generated by the SALSA compiler, do not modify.\n";
		code += "import java.io.IOException;\n";
		code += "import java.util.Vector;\n\n";
		code += "import java.lang.reflect.Method;\n";
		code += "import java.lang.reflect.Modifier;\n";
		code += "import java.lang.reflect.InvocationTargetException;\n\n";

		code += "import salsa.language.Actor;\n";
		code += "import salsa.language.ActorReference;\n";
		code += "import salsa.language.Message;\n";
		code += "import salsa.language.RunTime;\n";
		code += "import salsa.language.ServiceFactory;\n";
		code += "import gc.WeakReference;\n";

		if (getChild(children.length-1) instanceof BehaviorDeclaration) {
			code += "import salsa.language.Token;\n";
			code += "import salsa.language.exceptions.*;\n";
			code += "import salsa.language.exceptions.CurrentContinuationException;\n";

			String actorType = System.getProperty("actor");
			if (actorType == null) actorType = "salsa.language.UniversalActor";

			code += "\n";
			code += "import " + actorType + ";\n\n";
			code += "import salsa.naming.UAN;\n";
			code += "import salsa.naming.UAL;\n";
			code += "import salsa.naming.MalformedUALException;\n";
			code += "import salsa.naming.MalformedUANException;\n\n";
                        code += "import salsa.resources.SystemService;\n\n";
                        code += "import salsa.resources.ActorService;\n\n";
		}

		code += "// End SALSA compiler generated import delcarations.\n\n";

		for (int i = place; i < children.length; i++) {
			if (getChild(i) instanceof BehaviorDeclarationAttributes) code += "\n";
			code += getChild(i).getJavaCode();
		}

		SalsaCompiler.indent = 0;
		return code;
	}
}
